/* -*- c++ -*-
 * alex iliev, feb 2003
 * bit_matrix.h
 */


#ifndef _BIT_MATRIX_H
#define _BIT_MATRIX_H

#include <vector>

#include <stdlib.h>


typedef std::vector<std::bit_vector> bit_matrix;


// the information needed to access a slice of a bit_matrix
// a slice is a submatrix, from a given origin and of a given size
// assume we only need to carry the origin, and not the size
class matrix_slice_info_t {

public:
    
    matrix_slice_info_t () :
	start_col(0), start_row(0)
	{}

    // make a new one with new
    matrix_slice_info_t (const matrix_slice_info_t& orig,
			 size_t start_col, size_t start_row)
	: start_col(orig.start_col + start_col),
	  start_row(orig.start_row + start_row)
	{}

    size_t start_col, start_row;


};



//
// index into a slicable matrix
//
std::bit_vector::reference idx (bit_matrix & m,
				const matrix_slice_info_t& slice,
				size_t col, size_t row);

std::bit_vector::const_reference idx (const bit_matrix & m,
				      const matrix_slice_info_t& slice,
				      size_t col, size_t row);


//
// get a column of a slicable matrix
//
bit_matrix::reference column (bit_matrix & m,
			      const matrix_slice_info_t& slice,
			      size_t col);

bit_matrix::const_reference column (const bit_matrix & m,
				    const matrix_slice_info_t& slice,
				    size_t col);


//
// index into a slicable bit_vector
//
std::bit_vector::reference idx (std::bit_vector & v,
				const matrix_slice_info_t& slice,
				size_t row);

std::bit_vector::const_reference idx (const std::bit_vector & v,
				      const matrix_slice_info_t& slice,
				      size_t row);

#endif // _BIT_MATRIX_H
