/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * hostcall.cc
 * alex iliev, nov 2002
 */


#include <common/hostcall.h>
#include <common/exceptions.h>
#include <common/xdr_class.h>
#include <common/consts.h>
#include <common/socket-class.h>

#include <common/countptr.hpp>


#include "hostcall.h"


using namespace std;



static host_status_t
send_host_request (ByteBuffer & reply, const ByteBuffer& req)
    throw (comm_exception);

static void make_blob_address (blob_address_x & addr,
			       const std::string& basedir,
			       const object_name_t& name);


#ifdef TESTING_HOSTCALL

#include <fstream>
#include <stdlib.h>
#include <iostream>

int main (int argc, char* argv[]) {

#ifdef TESTING_CLEAR_RECORD_FETCH
    
    // just run a clear record fetch

    if (argc < 3) {
	cerr << "Usage: " << argv[0] << " <bucket #> <record #>" << endl;
	exit (EXIT_FAILURE);
    }
    
    object_id id = { atoi(argv[1]),
		     atoi(argv[2]) };
    Record rec;

    host_status_t status = host_fetch_clearrecord (id, rec);
    if (status == STATUS_OK) {
	cout << "The record-->" << rec << "<--" << endl;
    }
    else {
	cerr << "fetch_clearrecord Host error " << STATUS_OK << endl;
    }

    // and now the same encrypted fetch
    blob encrec;
    status = host_fetch_record (id, encrec);
    if (status == STATUS_OK) {
	cout.write (encrec.bytes.cdata(), encrec.bytes.len());
    }
    else {
	cerr << "fetch_record Host error " << STATUS_OK << endl;
    }

#endif // TESTING_CLEAR_RECORD_FETCH


#ifdef TEST_NEW_IO

// use direct call to the host side, without sending packets
#define TOY_HOSTCALL

    string contents;
    ByteBuffer result;
    
    readfile (stdin, contents);

    clog << "The file -->" << contents << "<--" << endl;
    

    object_id id = { 4, 5 };
    
    try {
	host_write_blob (std::string("test-dir"),
			 id,
			 ByteBuffer (contents));
	
	host_read_blob (std::string("test-dir"),
			id,
			result);
    }
    catch (const better_exception & ex) {
	cerr << ex.what() << endl;
	exit (1);
    }
    
    cout << "The result:" << endl;
    cout.write (result.cdata(), result.len());

#endif // TEST_NEW_IO

}


#endif // TESTING_HOSTCALL





//
// the user service functions
//

#if 0
host_status_t host_fetch_blob (const object_id& id,
			       blob & o_result)
{
    host_status_t answer;

    // serialize param
    XDR_STRUCT(object_id) xdrcall (id);
    ByteBuffer param = xdrcall.encode();
    ByteBuffer result;
    
    answer = invoke_host (HOST_FETCH_BLOB, param, result);

    if (answer == STATUS_OK) {
	o_result.reconstruct (result);
    }

    return answer;
}


host_status_t host_fetch_clearrecord (const object_id& id,
				      Record & o_result)
{
    host_status_t answer;

    // serialize param
    XDR_STRUCT(object_id) xdrcall (id);
    ByteBuffer param = xdrcall.encode();
    ByteBuffer result;
    
    answer = invoke_host (HOST_FETCH_CLEARRECORD, param, result);

    if (answer == STATUS_OK) {
#ifdef TESTING_HOSTCALL
	ofstream ofs ("record-card.bin");
	ofs.write (result.cdata(), result.len());
	ofs.close();
#endif
	o_result.reconstruct (result);
    }

    return answer;
}
#endif


#if 0
host_status_t host_write_blob (const named_blob& rec)
{
    host_status_t answer;

    ByteBuffer param = rec.serialize();
    ByteBuffer result;

    answer = invoke_host (HOST_WRITE_BLOB, param, result);

    return answer;
}
#endif



//
// the new blob IO functions
//


// read object named with an object_id
void host_read_blob (const std::string& basedir,
		     const object_name_t& name,
		     ByteBuffer & o_result)
    throw (host_exception, xdr_exception, comm_exception)
{
    // need to generate a blob_address_x, and then reconstruct a ByteBuffer

    // prepare the parameters
    XDR_STRUCT(blob_address_x) xdrarg;
    make_blob_address (xdrarg.x, basedir, name);

    ByteBuffer bytearg = xdrarg.encode();
    ByteBuffer byte_answer;

    host_status_t status = invoke_host (HOST_READ_BLOB_NEW,
					bytearg, byte_answer);
    if (status != STATUS_OK) {
	throw host_exception ("host_read_blob", status);
    }

    // construct the answer
    o_result = byte_answer;
}


void host_write_blob (const std::string& basedir,
		      const object_name_t& name,
		      const ByteBuffer & blob)
    throw (host_exception, xdr_exception, comm_exception)
{

    blob_to_write_x bx;
    make_blob_address (bx.address, basedir, name);
    bx.blob = blob.to_xdr();

//    XDR_STRUCT(blob_to_write_x) xdr (bx);
    ByteBuffer bytearg = XDR_STRUCT(blob_to_write_x) (bx).encode();
    
    ByteBuffer byte_answer;

    host_status_t status = invoke_host (HOST_WRITE_BLOB_NEW,
					bytearg, byte_answer);
    if (status != STATUS_OK) {
	throw host_exception ("host_write_blob", status);
    }

}







host_status_t host_notice (host_notice_t notice)
{
    host_status_t answer;
    XDR_STRUCT(host_notice_t) xdrcall (notice);
    ByteBuffer param = xdrcall.encode();
    ByteBuffer result;

    answer = invoke_host (HOST_NOTICE, param, result);

    return answer;
}


static void make_blob_address (blob_address_x & addr,
			       const std::string& basedir,
			       const object_name_t& name)
{
    addr.basedir = const_cast<char*> (basedir.c_str());
    addr.name    = name;
}


static inline ByteBuffer make_call_buffer (host_service_id_t service,
					   const ByteBuffer& params)
    throw (xdr_exception)
{
    host_call_buffer callbuf = { service, params.to_xdr() };
    XDR_STRUCT(host_call_buffer) xdr (callbuf);
    return xdr.encode ();
//    return xdr_encode(host_call_buffer) (callbuf);
}


// 'host_answer' is the whole buffer returned by the host
// 'o_result' is the encoded version of the service return value (without the
// return status, which is returned)
static inline host_status_t unpack_return_buffer (const ByteBuffer& host_answer,
						  ByteBuffer & o_result)
    throw (xdr_exception)
{
    XDR_STRUCT(host_return_buffer) xdrret;
    xdrret.decode (host_answer);

    if (xdrret.x.status == STATUS_OK) {
	o_result = ByteBuffer (xdrret.x.result);
    }
    
    return xdrret.x.status;
}


//
// the function which will be used to invoke services on the host
//
host_status_t invoke_host (host_service_id_t service,
			   const ByteBuffer& params,
			   ByteBuffer& o_result)
    throw (xdr_exception, host_exception, comm_exception)
{
    ByteBuffer tosend = make_call_buffer(service, params);

    // send the request to the host...
    ByteBuffer answer;
    host_status_t status = send_host_request (answer, tosend);
    if (status != STATUS_OK) {
	perror ("Host communication failure");
	// TODO:
	// throw host_exception ("Invoking service", status);
	return status;
    }

    return unpack_return_buffer (answer, o_result);
}






// FIXME: better initialization would not be bad!
SCCDatagramSocket g_sock;
// address of the card server running on the host (card # 0)
SCCSocketAddress g_host_addr (0, PIR_HOSTSERV_PORT);



//
// for debugging, bring in the host's request handling function
//
#ifdef TOY_HOSTCALL
#include <host/hostcall.h>
#endif


static host_status_t
send_host_request (ByteBuffer & reply, const ByteBuffer& req)
    throw (comm_exception)
{

#ifdef TOY_HOSTCALL
    // keep it simple - just call the host function (which should have been
    // linked in)
    reply = host_handle_request (req);
    return STATUS_OK;
#endif
    
    host_status_t status = STATUS_OK;
    counted_ptr<SocketAddress> peer_addr;

    // send request
    g_sock.send (req, g_host_addr);

    reply = g_sock.recv (peer_addr);

    // TODO: maybe do something to check where the reply packet came from...

    return status;
}
