/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * card_server.cc
 * alex iliev, jan 2003
 * server to service requests from the card, eg to read and write records
 */

#include <iostream>

#include <unistd.h>

#include <stdlib.h>

#include <common/consts.h>
#include <common/utils.h>
#include <common/exceptions.h>
#include <common/socket-class.h>


#include "hostcall.h"



using namespace std;



static void usage (const char* progname);
static void reqloop (DatagramSocket & sock)
    throw (comm_exception);



int main (int argc, char *argv[]) {

    unsigned short port = PIR_HOSTSERV_PORT;


    int opt;
    while((opt = getopt(argc, argv, "p:")) != EOF) {
	switch(opt) {
	case('p'):
	    port = atoi(optarg); break;
	    
	default:
	    usage(argv[0]);
	    exit(EXIT_FAILURE); break;
	} /* switch options */
    }

	
    //
    // set up an SCC socket and loop waiting for requests
    //

    try {
	SCCDatagramSocket sock (port);
	reqloop (sock);
    }
    catch (const comm_exception& ex) {
	cerr << "Communication error: " << ex.what() << endl;
    }
}



static void usage (const char* progname) {
    cerr << "Usage: " << progname << ": [-p port]" << endl;
}


static void reqloop (DatagramSocket & sock)
    throw (comm_exception)
{

    counted_ptr<SocketAddress> client_addr;
    
    while(true) {

	/* get request */
	ByteBuffer req    = sock.recv (client_addr);
	
	try {
	    ByteBuffer answer = host_handle_request (req);
	    sock.send (answer, *client_addr);
	}
	catch (const xdr_exception& ex) {
	    // FIXME: the error should be communicated to the client
	    cerr << "XDR exception: " << ex.what() << endl;
	}

    }

}
