/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

/*
 * host/hostcall.cc
 * alex iliev, nov 2002
 */

#include <fstream>
#include <iostream>
#include <sstream>
#include <stdexcept>


// these 3 for stat() and mkdir()
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <time.h>		// for nanosleep

#include <common/utils.h>
#include <common/hostcall.h>
#include <common/xdr_class.h>
#include <common/record.h>
#include <common/comm_types.h>
#include <common/consts.h>

#include "hostcall.h"


using namespace std;


// FIXME: these should not be in the source like this!
static const std::string CRYPTROOT = "/peanuts/priv-db/crypt",
    TEMP_CRYPTROOT = "/peanuts/priv-db/temp-crypt",
    CLEARROOT = "/peanuts/priv-db/clear";




    
//
// little helpers
//

static ByteBuffer
fetch_blob (const std::string & filename)
    throw (std::ios::failure);


// make the filename for an object given its name as a object_name_t
static std::string get_object_filename (const std::string& basedir,
					const object_name_t& name)
    throw (std::invalid_argument);


// write 'blob' into 'filename'
host_status_t host_write_blob (const ByteBuffer& blob,
			       const std::string & filename);


// static bool
// dir_exists (const std::string& name) throw (std::ios_base::failure);




//
// the handling functions, same prototypes as in the card (just accidental)
// keep them out of sight of other code!
//

static host_status_t host_read_blob_new (const std::string& basedir,
					 const object_name_t& id,
					 ByteBuffer & o_result);

static host_status_t host_write_blob_new (const std::string& basedir,
					  const object_name_t& id,
					  const ByteBuffer & data);

#if 0
static host_status_t host_fetch_blob (const object_id& id,
					blob & o_result);

static host_status_t host_fetch_clearrecord (const object_id&,
					     Record & o_result);

static host_status_t host_write_blob (const named_blob&,
				      const std::string& dir);
#endif


static host_status_t take_notice (const host_notice_t& notice);




#ifdef TESTING_HOSTCALL

int main () {
    
#ifdef TESTING_CLEAR_RECORD
    Record rec;

    try {

	for (index_t i = 0; i < 3000; i++) {
	    fetch_clear_record (i, 0, rec);
	    cout << "second attrib: " << rec.attributes.front().name
		 << " = " << rec.attributes.front().value << endl;
	}

    }
    catch (const ios::failure & ex) {
	cerr << "Exception: " << ex.what() << endl;
	exit (1);
    }
#endif // TESTING_CLEAR_RECORD


    

}

#endif // TESTING_HOSTCALL


ByteBuffer host_handle_request (const ByteBuffer& req)
    throw (xdr_exception)
{

    XDR_STRUCT(host_call_buffer) xdrreq;
    xdrreq.decode (req);

    host_call_buffer & callbuf = xdrreq.x;
    

    host_service_id_t servid = xdrreq.x.service_id;

    // xdrreq.x.params is the ByteBuffer_x with the parameters to the request


    ByteBuffer answer;
    host_status_t status = STATUS_OK;
    
    switch (servid) {

    case HOST_READ_BLOB_NEW:
    {
	XDR_STRUCT(blob_address_x) xdrparam;
	xdrparam.decode (callbuf.params);
	blob_address_x & address = xdrparam.x;

	status = host_read_blob_new (address.basedir, address.name, answer);
    }
    break;


    case HOST_WRITE_BLOB_NEW:
    {
	XDR_STRUCT(blob_to_write_x) xdrparam;
	xdrparam.decode (callbuf.params);

	status = host_write_blob_new (xdrparam.x.address.basedir,
				      xdrparam.x.address.name,
				      xdrparam.x.blob);
    }
    break;
    
    
#if 0
    case HOST_FETCH_BLOB:
    {
	XDR_STRUCT(object_id) xdrparam;
	blob encrec;

	// xdrreq.x.params is of type ByteBuffer_x
	ByteBuffer paramsbuf (xdrreq.x.params);
	xdrparam.decode (paramsbuf);
	
	// do the work
	status = host_fetch_blob (xdrparam.x, encrec);

	if (status == STATUS_OK) {
	    answer = encrec.serialize();
	}
    }
    break;
	
    case HOST_FETCH_CLEARRECORD:
    {
	XDR_STRUCT(object_id) xdrparam;
	Record rec;

	xdrparam.decode (xdrreq.x.params);
	
	// do the work
	status = host_fetch_clearrecord (xdrparam.x, rec);

	if (status == STATUS_OK) {
	    answer = rec.serialize();

// 	    ofstream ofs ("record-host.bin");
// 	    ofs.write (answer.cdata(), answer.len());
// 	    ofs.close();
	}
    }
    break;

    case HOST_WRITE_BLOB:
    {
	// do the work
	named_blob r (ByteBuffer (xdrreq.x.params)); // deserialize
	std::ostringstream filename;
	
	status = host_write_blob (r, CRYPTROOT);

	// 'answer' will be left empty here
    }
    break;
#endif // 0

    case HOST_NOTICE:
    {
	XDR_STRUCT(host_notice_t) xdrparam;
	xdrparam.decode (xdrreq.x.params);

	// now the notice is in xdrparam.x
	status = take_notice (xdrparam.x);
    }
    break;

    default:
	status = STATUS_BAD_PARAM;
	break;
    

    } // end switch

    // construct the return buffer

    host_return_buffer retbuf = { status, answer.to_xdr() };
    
    XDR_STRUCT(host_return_buffer) xdrret (retbuf);
    return xdrret.encode();
}






inline static void delay () {
    return;
#if 0
    struct timespec t;
    t.tv_sec = 0;
    t.tv_nsec = 100000000;	// 100,000,000 nsec = 1/10 second
    nanosleep (&t, NULL);
#endif
}



//
// the "new" IO functions
//


static host_status_t host_read_blob_new (const std::string& basedir,
					 const object_name_t& name,
					 ByteBuffer & o_result)
{
    try {
	std::string filename = get_object_filename (basedir, name);
	o_result = fetch_blob (filename);
    }
    catch (const std::invalid_argument& ex) {
	return STATUS_BAD_PARAM;
    }
    catch (const std::ios_base::failure & ex) {
	cerr << "Exception while fetching blob: " << ex.what() << endl;
	return STATUS_IO_ERROR;
    }
    
    return STATUS_OK;
}


static host_status_t host_write_blob_new (const std::string& basedir,
					  const object_name_t& name,
					  const ByteBuffer & data)
{
    
    try {
	std::string filename = get_object_filename (basedir, name);
	return host_write_blob (data, filename);
    }
    catch (const std::invalid_argument& ex) {
	return STATUS_BAD_PARAM;
    }
}


static std::string get_object_filename (const std::string& basedir,
					const object_name_t& name)
    throw (std::invalid_argument)
{
    std::ostringstream filename;

    filename << STOREROOT << DIRSEP << basedir << DIRSEP;
    
    // add the object name, which is for now an object_id or just a string
    
    switch (name.name_scheme) {
    case NAME_OBJECT_ID:
	filename << name.object_name_t_u.oid_name.bucket
		 << DIRSEP << name.object_name_t_u.oid_name.record << ends;
	break;
    case NAME_STRING:
	filename << name.object_name_t_u.str_name << ends;
	break;

    default:
	cerr << "Wrong name scheme " << name.name_scheme << endl;
	throw std::invalid_argument ("Unknown object name scheme");
    }

    return filename.str();
    
}

#if 0
static host_status_t host_fetch_blob (const object_id& id,
				      blob& o_result)
{
    //
    // for now we can organize the records in directories so we will fetch
    // file $ROOT/bucket/record
    //

    
    try {
	o_result.bytes = fetch_blob (id.bucket, id.record, CRYPTROOT);
    }
    catch (const ios::failure & ex) {
	cerr << "Exception while fetching blob: " << ex.what() << endl;
	return STATUS_IO_ERROR;
    }
    
    return STATUS_OK;
}



static host_status_t host_fetch_clearrecord (const object_id& id,
					     Record & o_result)
{
    
    Record rec;
    try {
	fetch_clear_record (id.bucket, id.record, o_result);
    }
    catch (const ios::failure & ex) {
	cerr << "fetch_clear_record failed: " << ex.what() << endl;
	return STATUS_IO_ERROR;
    }

    return STATUS_OK;
}
#endif // 0


host_status_t host_write_blob (const ByteBuffer& blob,
			       const std::string & filename)
{

    int rc;
    
    static size_t calls = 0;
    if (++calls % 100 == 0) {
	clog << "write " << filename << endl;
    }
    
    std::ofstream ofs;
    ofs.sync_with_stdio (false);
    
    ofs.open (filename.c_str(), std::ios::out | std::ios::trunc);
    
    if (!ofs) {
	    
	if (errno != ENOENT) {
	    // some other error, give up
	    perror ("Bad open error");
	    return STATUS_IO_ERROR;
	}

	// try build the parent directories
	rc = builddirs (filename, 00700);
	if (rc != 0) {
	    perror ("mkdir error");
	    return STATUS_DIR_ERROR;
	}
	
	// redo the open
	ofs.clear();
	ofs.open (filename.c_str(), std::ios::out | std::ios::trunc);
	if (!ofs) {
	    perror ("Error opening record file");
	    return STATUS_IO_ERROR;
	}
    }

	
    ofs.write (blob.cdata(), blob.len());
    if (ofs.fail()) {
	perror ("Error writing record bytes");
	return STATUS_IO_ERROR;
    }
	
    return STATUS_OK;
}




//
// little helpers
//

#if 0
bool dir_exists (const std::string& name) throw (std::ios_base::failure) {

    struct stat st;
    memset (&st, 0, sizeof(st));

    clog << "Checking dir " << name << endl;
    
    if (stat (name.c_str(), &st) != 0) {
	return false;
    }

    if (S_ISDIR(st.st_mode)) {
	return true;
    }

    // and now?
    // FIXME: invent an excpetion hier. especially for these host calls
    throw std::ios::failure ("Dir name exists but is not a directory");
}
#endif // 0



ByteBuffer
fetch_blob (const std::string & filename)
    throw (std::ios::failure)
{

    std::string contents;
    FILE * fh = fopen (filename.c_str(), "r");
    if (!fh) {
	throw ios::failure ("File open failed: " +
			    std::string (strerror(errno)));
    }

    readfile (fh, contents);
    fclose (fh);

    // need a deep copy of the string
    ByteBuffer bytes (new byte[contents.length()], contents.length());
    contents.copy (bytes.cdata(), bytes.len());

    return bytes;
}



#if 0
static void
open_rec_file (index_t bucket, index_t record, const std::string& root,
	       ifstream & o_ifs)
    throw (ios::failure)
{
    ostringstream filename;
    
    filename << CLEARROOT << DIRSEP << bucket << DIRSEP << record;

    o_ifs.open (filename.str().c_str());
    if (!o_ifs) {
	cerr << "Failed to open " << filename.str() << endl;
    }
}

void
fetch_clear_record (index_t bucket, index_t record,
		    Record & o_rec)
    throw (ios::failure)
{
    ifstream ifs;
    
    // FIXME: this exception stuff doesnt seem to work as advertised. abandon
    // for now
    ifs.exceptions (ifstream::badbit);

    open_rec_file (bucket, record, CLEARROOT, ifs);
    if (!ifs) {
	throw ios::failure ("Failed to open record file for reading");
    }

    
    // the name
    getline (ifs, o_rec.name);
    
    // and now the attributes
    RecordAttr current;
    
    while (getline (ifs, current.name) && getline (ifs, current.value)) {
	o_rec.attributes.push_back (current);
    }
}
#endif


static host_status_t take_notice (const host_notice_t& notice) {

    host_status_t status = STATUS_OK;

    switch (notice) {
    case NOTICE_SHUFFLE_DONE:
	// we want to move the encrypted DB under crypt-temp/ to crypt/
	if ( rename (TEMP_CRYPTROOT.c_str(), CRYPTROOT.c_str()) != 0 ) {
	    perror ("Moving temp-crypt dir to crypt dir");
	    status = STATUS_DIR_ERROR;
	}

	break;

    default:
	status = STATUS_BAD_PARAM;
	break;
    }

    return status;

}
