/*
 * ** PIR Private Directory Service prototype
 * ** Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
 * **
 * ** This program is free software; you can redistribute it and/or modify
 * ** it under the terms of the GNU General Public License as published by
 * ** the Free Software Foundation; either version 2 of the License, or
 * ** (at your option) any later version.
 * **
 * ** This program is distributed in the hope that it will be useful,
 * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * ** GNU General Public License for more details.
 * **
 * ** You should have received a copy of the GNU General Public License
 * ** along with this program; if not, write to the Free Software
 * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * */

// makerecords.cc
// alex iliev jan 2003

// code to make and hash records for the private directory

// reads a list of names on STDIN and writes partially random records for them
// inside STOREROOT/clear



#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <vector>

#include <stdlib.h>

#include <common/record.h>
#include <common/hash.h>
#include <common/consts.h>


using namespace std;


Record make_record (const string& name);
void write_rec (const Record& rec, const string& filename);


int main (int argc, char * argv[]) {

    string name;
    size_t dbsize;

    size_t max_size = 0;

    index_t i=0;

    if (argc < 2) {
	cerr << "Usage: " << argv[0] << " <db size>" << endl;
	exit (EXIT_FAILURE);
    }

    dbsize = atoi (argv[1]);
    
    vector<index_t> bucket_sizes (dbsize, 0);

    while (getline (cin, name)) {

	ostringstream filename;

	index_t hashed = hash
	    (reinterpret_cast<const unsigned char*> (name.c_str()), dbsize);

	filename << STOREROOT << "/" << "clear" << "/" << hashed;
	system ( ("mkdir -p " + filename.str()).c_str() );

	filename << "/" << bucket_sizes[hashed];
	bucket_sizes[hashed] ++;

	max_size = max (max_size, bucket_sizes[hashed]);

	Record r = make_record (name);
	write_rec (r, filename.str());

	i++;
    }

    if (i != dbsize) {
	cerr << "Warning: actually made " << i << " records, not "
	     << dbsize << " as specified" <<  endl;
    }

    //
    // now need to add filler records to each bucket so they all have the same
    // size

    // just make it with a constant name for now - can use more random names
    // later

    clog << "Starting to write filler records: "
	 << max_size << " reocrds per bucket" << endl;
    
    for (i=0; i < dbsize; i++) {

	ostringstream dir;
	dir << STOREROOT << "/" << "clear" << "/" << i;
	system ( ("mkdir -p " + dir.str()).c_str() );

	for (index_t j = bucket_sizes[i]; j < max_size; j++) {

	    ostringstream  filename;
	    
	    filename << dir.str() << "/" << j;
	    Record r = make_record("Anton Dziribiewski");

	    write_rec (r, filename.str());
	    
	}

    }

}


// writes a human readable version of the record in <filename>.txt
// and the XDR form in just <filename>
void write_rec (const Record& rec, const string& filename) {
    ofstream ofs ((filename + ".txt").c_str());
    if (!ofs) {
	cerr << "Problem opening " << filename + ".txt" << endl;
	abort();
    }
    ofstream ofsxdr (filename.c_str());
    if (!ofsxdr) {
	cerr << "Problem opening " << filename << endl;
	abort();
    }
    
    ofs << rec;

    ByteBuffer enc = rec.serialize();
    ofsxdr.write (reinterpret_cast<const char*> (enc.data()), enc.len());
}


static char BASE64_TABLE [] =
"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
				

Record make_record (const string& name) {

    Record r;
    r.name = name;

    ostringstream os;

    r.attributes.push_back (RecordAttr ("cn", name));
    
    os << 2000 + random() % 10000;
    string uid = os.str(); os.str ("");
    r.attributes.push_back (RecordAttr ("uid", uid));

    istringstream is_name (name);
    string givenName, sn, middle;
    is_name >> givenName >> middle >> sn;
    if (sn.length() == 0)
	sn = middle; // didnt have a middle name it seems
    r.attributes.push_back (RecordAttr ("givenName", givenName));
    r.attributes.push_back (RecordAttr ("sn", sn));
    r.attributes.push_back (RecordAttr ("initials", middle));

    sn[0] = tolower(sn[0]);
    r.attributes.push_back (RecordAttr ("mail", sn+"@dartmoor.edu"));

#if 0
    // some DND fields for fun!
    r.attributes.push_back (RecordAttr ("dndaffiliation", "DART"));
    r.attributes.push_back (RecordAttr ("dndbullserv",
					"newshost.dartmouth.edu@internet"));

    os << random();
    r.attributes.push_back (RecordAttr ("dnddartfilesquo", os.str()));
    os.str("");
    
    r.attributes.push_back (RecordAttr ("dnddatasource", "BAN"));
    r.attributes.push_back (RecordAttr ("dnddctsnum", "GF45823B"));
    r.attributes.push_back (RecordAttr ("dnddeptclass", "GR"));
    r.attributes.push_back (RecordAttr ("dndpotluckpath",
					sn.substr (0,1) + ":" +
					sn.substr (0,2) + ":" +
					sn.substr (0,3) + ":" +
					name));

    r.attributes.push_back (RecordAttr ("dndpotluckserv", "Vault"));
    r.attributes.push_back (RecordAttr ("dndstuff", "some crap"));
    r.attributes.push_back (RecordAttr ("dnduid", uid));
    r.attributes.push_back (RecordAttr ("krbname", name+"@dartmoor.edu"));
    r.attributes.push_back (RecordAttr ("displayname", name));
    r.attributes.push_back (RecordAttr ("edupersonorgdn",
					"dc=dartmoor, dc=edu"));
    
    os << random();
    r.attributes.push_back (RecordAttr ("dndgid", os.str()));
    os.str("");
    
    r.attributes.push_back (RecordAttr ("dndblitzinfo", "/vixen0"));
    r.attributes.push_back (RecordAttr ("dndblitzserv",
					"newvixen.Dartmouth.EDU@BlitzServ"));
    r.attributes.push_back (RecordAttr ("nickname", "nick nolte"));

    
    os << "HB " << random() % 5000;
    r.attributes.push_back (RecordAttr ("dndhinmanaddr", os.str()));
    os.str("");

    r.attributes.push_back (RecordAttr ("telephonenumber", "1-800-DEADBEEF"));
    r.attributes.push_back (RecordAttr ("dndurl",
					"http://www.dartmoor.edu/~moor"));
			    
    r.attributes.push_back (RecordAttr ("dndmailaddr",
					name+"@cs.dartmoor.edu"));
    
#endif // 0
    

    // make some random BASE64 stringa as pretend certificates
    // the size of a base64 cert in the DND was 1250 characters or so
    const size_t CERTSIZE = 150; //1250
    for (int j = 0; j < 1; j++) {
	os.str ("");	// reset the string to empty
	for (size_t i=0; i < CERTSIZE; i++) {
	    os << BASE64_TABLE[random() % 64];
	}
	char certnum[] = {j+'1', '\0'};
	r.attributes.push_back (RecordAttr
				(string("usercertificate;binary") + certnum,
				 os.str()));
    }
    

    os.str("");
    os << setprecision(3) << 1.5 + (random() % 10000) / 20003.0;
    r.attributes.push_back (RecordAttr ("height", os.str()));

    return r;
}


