import Posix				-- for popen, this is apparently
					-- deprecated, but i don't see any
					-- replacement :(

import System
import IO
import Array
import Random
import List
import Maybe
import Time
import Locale(defaultTimeLocale)
import Char(ord)

import Concurrent


-- the exponential density parameter
g_lambda = 1/15




main :: IO ()
main = do -- get the cmd line args
	  (nameFile:avgPeriod:_) <- getArgs
	  -- get the names used in the DB, as an array
	  names       <- getNames nameFile
	  -- prepare two (hopefully independent) random generators
	  (gen1,gen2) <- getDevRandomGen >>= (return . split)
	  -- run an infinite number of oneQuery's on a random stream of names
	  mapM_ (oneQuery (1/(read avgPeriod)))
		(zip3 (randomRs (0,1) gen1) (getRandomItems gen2 names) [1..])




-- get a nice random generator invoking the help of /dev/urandom
-- FIXME: this leaves the file handle unclosed, not sure what to do about that
getDevRandomGen :: IO StdGen
getDevRandomGen =
    do randStr <- openFile "/dev/urandom" ReadMode >>= hGetContents
       return (mkStdGen (intFromBin randStr))
	      

-- get an integer from the head of a binary string
intFromBin :: String -> Int
intFromBin bin = foldl addExp 0 (zip (map ord (take 4 bin)) [24,16,8,0] )
    where addExp part (b,e) = part + b*(2^e)


-- get the array of names we'll be using
getNames :: String -> IO (Array Int String)
getNames fileName =
    do nameString <- openFile fileName ReadMode >>= hGetContents
       return $ let nameList = lines nameString
		in  listArray (0, (length nameList)-1) nameList


-- rnd is a uniform random real number between 0 and 1
oneQuery :: Float -> (Float,String,Int) -> IO ()
oneQuery lambda (rnd,name,num) = do let waitTime = getExpRandom lambda rnd
				    forkIO (doQuery stderr name num)
				    -- wait for random amount
  				    threadDelay (ceiling (waitTime * 1000000))
				    return ()

-- resfh: where to put the query results
doQuery resfh name n =
    do putStrLn $ openTag "query" [("num",n)]
       printTime "time-start" n
       putStrLn $ makeTag "name" name [("num",n)]
       hFlush stdout
       (out,err) <- sendQuery name
       putStrLn $ makeTag "output" ('\n':out ++ "\n") [("num",n)]
       putStrLn $ makeTag "errors" ('\n':err ++ "\n") [("num",n)]
       printTime "time-done" n
       putStrLn $ closeTag "query"
       putStr "\n\n"
       hFlush stdout



printTime name n = do calTime <- getClockTime >>= toCalendarTime
		      putStrLn (makeTag name (myFormat calTime) [("num",n)])
    where myFormat cal =
	      formatCalendarTime defaultTimeLocale "%Y-%m-%d-%H-%M-%S." cal ++
	      show (div (ctPicosec cal) 1000000)

--
-- some quick XML tag generation routines
--
makeTag name val attrs = openTag name attrs ++ val ++ closeTag name
-- open a tag with given name and list of attributes
openTag :: Show a => String -> [(String,a)] -> String
openTag name attrs = "<"  ++ name ++ foldl addAttr "" attrs ++ ">"
    where addAttr part (name,val) =
	      concat [part, " ", name, "=\"", show val, "\""]

closeTag name = "</" ++ name ++ ">"




sendQuery :: String -> IO (String,String)
sendQuery name =
    do (out, err, pid) <- popen "ldapsearch"
			  ["-h", "129.170.19.223",
			   "-x",
			   "-b", "dc=brazil,dc=pir",
			   "(cn=" ++ name ++ ")", -- the filter
			   "ALL"]	          -- attributes to return
			  Nothing
       getProcessStatus True False pid	-- reap the process
       return (out,err)

--     do (out, err, pid) <- popen "ldapsearch"
-- 			  ["-h", "dnd.dartmouth.edu",
-- 			   "-x",
-- 			   "-b", "dc=dartmouth,dc=edu",
-- 			   "(cn=*" ++ name ++ "*)", -- the filter
-- 			   "cn"]	            -- attributes to return
-- 			  Nothing



-- this takes a uniform random real number between 0 and 1
-- lambda is the density parameter (inv. of the mean)
getExpRandom :: Float -> Float -> Float
getExpRandom lambda = (* (-1/lambda)) . log



-- return a stream of randomly chosen items from the array 'arr'
getRandomItems :: (RandomGen g) => g -> Array Int b -> [b]
getRandomItems gen arr = (randomItem : getRandomItems newgen arr)
    where (randomItem,newgen) = let size             = (rangeSize . bounds) arr
				    (rIndex, newgen) = randomR (0,size-1) gen
				in ((arr ! rIndex) , newgen)



-- a list of the partial applications of f to the sublists of the given list,
-- starting with value 'zero'
partialOp :: (a -> b -> a) -> a -> [b] -> [a]
partialOp f zero     = snd . mapAccumL (\s -> \x -> (f s x,f s x)) zero
