#!/usr/bin/perl -w

## Copyright (C) 2002 Alexander Iliev <iliev@nimbus.dartmouth.edu>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


use English;
use IPC::Open3;


$DIR="$ENV{HOME}/work/shib/code/host";
$PIRSEARCH="$DIR/pirsearch";
$CARDNO = 2;

open LOG, ">>$DIR/logs/pir-query-$$.log" or die "opening log: $ERRNO";

print LOG "Reading the LDAP operation\n";
# read the operation requested. should be SEARCH
my $op = <>; chomp $op;
if ($op ne 'SEARCH') { die "Wrong operation: $op"; }

print LOG "Reading the LDAP parameters\n";

# read the parameters from LDAP, which are a tag name followed by a space and
# the tag value

my %req;
while (<>) {
  chomp;

  # lines are of the form
  # tag: value
  my ($tag, $value) = /([[:lower:]]+): (.*)/;
  $req{$tag} = $value
    if defined $tag and defined $value;

}

print LOG "Opening pirsearch\n";


# open the actual PIR program
my($pid, $pirout, $pirin, $pirerr);
eval {
  $pid = open3 ($pirin, $pirout, $pirerr, $PIRSEARCH, $CARDNO);
};

die "open3 pirsearch: $EVAL_ERROR" if $EVAL_ERROR =~ /^open3:/;


print LOG "Writing the name to pirsearch\n";

# what are we searching for - write it to the search program
my ($target) = $req{filter} =~ /\(.*=(.*)\)/;
print $pirin $target, "\n";


# trap SIGPIPE in case the open did not work
$SIG{PIPE} = sub {
  die "SIGPIPE from pirsearch";
};



my ($recname, $name, $value);

print LOG "Reading record name\n";

$recname = <$pirout>; chomp $recname;

print <<EOF;
dn: cn=$recname,$req{base}
objectclass: top
objectclass: person
cn: $recname
EOF

print LOG "Reading name-value attribute list and printing it\n";

while (defined ($name  = <$pirout>) and
       defined ($value = <$pirout>))
  {
    chomp ($name, $value);
    print "$name: $value\n";
  }

# print the closing comments. note the blank line in there
print<<EOF;

RESULT
code: 0
EOF

print LOG "waitpid for pirsearch\n";

waitpid $pid, 0;

print LOG "done\n\n\n";

close LOG;

